<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class HierarquiaUsuariosFactory
{

    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    //Busca lista de Hierarquias cadastradas
    public function getHierarquia()
    {

        $sql = "SELECT NRHIERARQUIAFUNC, NMHIERARQUIAFUNC, IDATIVO
                 FROM FCM_HIERARQUIAFUNC";


        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    // Busca os usuários da hierarquia escolhida
    public function getHierarquiaDetalhes($param)
    {
        $sql = "SELECT A.NRFUNCIONARIOSHIERARQ, A.CDOPERADOR, A.CDOPERADORPAI, B.NMOPERADOR, H.NMHIERARQUIAFUNC
        FROM FCM_FUNCIONARIOSHIERARQ A
        JOIN OPERADOR B ON B.CDOPERADOR = A.CDOPERADOR
        JOIN FCM_HIERARQUIAFUNC H ON H.NRHIERARQUIAFUNC = A.NRHIERARQUIAFUNC
       WHERE A.NRHIERARQUIAFUNC = :NRHIERARQUIAFUNC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $param);
    }

    public function getHierarquiaFacilities($parametros)
    {

        $sql = "SELECT A.NRFUNCIONARIOSHIERARQ, A.NRFUNCIONARIO, A.NRFUNCIONARIOPAI, B.NMFUNCIONARIO, H.NMHIERARQUIAFUNC
                    FROM FCM_FUNCIONARIOSHIERARQ A
                        JOIN FCM_FUNCIONARIO B ON B.NRFUNCIONARIO = A.NRFUNCIONARIO
                        JOIN FCM_HIERARQUIAFUNC H ON H.NRHIERARQUIAFUNC = A.NRHIERARQUIAFUNC
                    WHERE A.NRHIERARQUIAFUNC = (SELECT NRHIERARQUIAFUNC
                                                FROM FCM_HIERARQUIAFUNC
                                                    WHERE IDATIVO = 'S')
                    AND A.NRFUNCIONARIO IN (SELECT NRFUNCIONARIO
                                                        FROM FCM_FUNCIONARIOSHIERARQ
                                                    START WITH NRFUNCIONARIO     = :NRFUNCIONARIO
                                                    CONNECT BY PRIOR NRFUNCIONARIO = NRFUNCIONARIOPAI)";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    // Busca hierarquia de usuários que podem ser cadastrados.
    public function getHierarquiaUsuarios($param)
    {
        $sql = " SELECT CDOPERADOR, NMFUNCIONARIO
                    FROM FCM_FUNCIONARIO
                  WHERE CDOPERADOR NOT IN( SELECT A.CDOPERADOR
                                                            FROM FCM_FUNCIONARIO A
                                                                JOIN FCM_FUNCIONARIOSHIERARQ B ON  B.CDOPERADOR = A.CDOPERADOR
                                                WHERE B.NRHIERARQUIAFUNC = :NRHIERARQUIAFUNC)";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $param);
    }

    // Busca uma lista personalizada de usuários que podem ser Pais na hierarquia
    public function getHierarquiaPais($param)
    {
        $sql = "SELECT CDOPERADOR, NMOPERADOR
                    FROM OPERADOR
                  WHERE CDOPERADOR IN( SELECT A.CDOPERADOR
                                                            FROM OPERADOR A
                                                                JOIN FCM_FUNCIONARIOSHIERARQ B ON  B.CDOPERADOR = A.CDOPERADOR
                                              WHERE B.NRHIERARQUIAFUNC = :NRHIERARQUIAFUNC)";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $param);
    }

    // Cadastra novas Hierarquias
    public function salvarHierarquia($params)
    {

        $sql = "INSERT INTO FCM_HIERARQUIAFUNC(NRHIERARQUIAFUNC, NMHIERARQUIAFUNC, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, CDOPERINCLUSAO, NRORGINCLUSAO,  CDOPERULTATU, NRORGULTATU)
                VALUES (:NRHIERARQUIAFUNC, :NMHIERARQUIAFUNC, :NRORG, :IDATIVO, SYSDATE, SYSDATE,  :CDOPERINCLUSAO, :NRORGINCLUSAO, :CDOPERULTATU, :NRORGULTATU)";


        $this->connection->executeStatement($sql, $params);
    }
    // Edita Hierarquias já cadastradas
    public function editarHierarquia($params)
    {
        $sql = "UPDATE  FCM_HIERARQUIAFUNC
                SET IDATIVO             = :IDATIVO, 
                NMHIERARQUIAFUNC        = :NMHIERARQUIAFUNC, 
                NRORGULTATU             = :NRORGULTATU, 
                CDOPERULTATU            = :CDOPERULTATU,
                DTULTATU                = SYSDATE
                WHERE NRHIERARQUIAFUNC  = :NRHIERARQUIAFUNC";


        $this->connection->executeStatement($sql, $params);
    }
    // Deleta hierarquias cadastradas.
    public function excluirHierarquia($parametro)
    {

        $sql = "DELETE FROM FCM_HIERARQUIAFUNC
                WHERE NRHIERARQUIAFUNC IN (" . $parametro . ")";

        $this->connection->executeStatement($sql);
    }

    // Insere um novo usuário à hierarquia
    public function inserirUsuarios($params)
    {
        $sql = "INSERT INTO FCM_FUNCIONARIOSHIERARQ (NRFUNCIONARIOSHIERARQ, NRHIERARQUIAFUNC, CDOPERADOR, CDOPERADORPAI,   NRORG, NRORGINCLUSAO, CDOPERINCLUSAO, DTINCLUSAO)
                                             VALUES (:NRFUNCIONARIOSHIERARQ, :NRHIERARQUIAFUNC, :CDOPERADOR, :CDOPERADORPAI, :NRORG, :NRORGINCLUSAO, :CDOPERINCLUSAO,  SYSDATE)";

        $this->connection->executeStatement($sql, $params);
    }

    // Deleta um usuário e sua ramificação abaixo
    public function excluirUsuarios($param)
    {

        $sql = "DELETE FROM FCM_FUNCIONARIOSHIERARQ
                    WHERE CDOPERADOR IN (" . $param . ")";

        $this->connection->executeStatement($sql);
    }

    public function buscarCdoperador($param)
    {
        $sql = "SELECT CDOPERADOR FROM FCM_FUNCIONARIOSHIERARQ 
        WHERE CDOPERADORPAI IN ( " . $param . " ) AND CDOPERADOR NOT IN ( " . $param . " )";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }
}
