<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class RelatoriosTarefasFactory
{

    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getRelatoriosTarefas($parametros, $operadores)
    {
        $session = "ALTER SESSION SET NLS_DATE_FORMAT = 'DD/MM/YYYY HH24:MI'";

        $this->connection->executeStatement($session);
        $sql = "SELECT CHKA.NRCHECKLISTAGEND, CHKA.NRLOCALCLIENTE, CHKA.NRCHECKLIST, CHKA.CDOPERADOR, CHKA.IDSITUACAOAGEND, 
                    CHKA.DTHORAAGEND, CHKA.IDATIVO, CHKA.DTHRFINALAGEND, CHKA.NMCHECKLISTAGEND, CONSULOBS.DSOBSTAREFAAGEND,
                    FUNC.NMOPERADOR, LC.NMLOCALCLIENTE, CHK.NMCHECKLIST, C.NMRAZSOCCLIE, ROUND(24 *(60 *(CHKA.DTHRFINALAGEND - CHKA.DTHORAAGEND)), 2) AS DURACAO, CHKA.DSJUSTCANCELAMENTO, LC.CDCLIENTE
                        FROM FCM_CHECKLISTAGEND CHKA
                    LEFT JOIN OPERADOR FUNC 
                        ON FUNC.CDOPERADOR = CHKA.CDOPERADOR
                    LEFT JOIN FCM_LOCALCLI LC
                        ON LC.NRLOCALCLIENTE = CHKA.NRLOCALCLIENTE
                    LEFT JOIN FCM_CHECKLIST CHK
                        ON CHK.NRCHECKLIST = CHKA.NRCHECKLIST
                    LEFT JOIN CLIENTE C
                        ON C.CDCLIENTE = LC.CDCLIENTE
                    LEFT JOIN (SELECT DISTINCT DSOBSTAREFAAGEND, NRCHECKLISTAGEND
                                    FROM FCM_TAREFAAGEND) CONSULOBS
                        ON CONSULOBS.NRCHECKLISTAGEND = CHKA.NRCHECKLISTAGEND
                      WHERE ((CHKA.CDOPERADOR IN ($operadores))
                         OR (CHKA.CDOPERINCLUSAO = :CDOPERADOR))";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDashRelatorioTarefas($parametros, $operadores)
    {
        $sql = "SELECT CHKA.NRCHECKLISTAGEND, CHKA.NRLOCALCLIENTE, CHKA.NRCHECKLIST, CHKA.CDOPERADOR, CHKA.IDSITUACAOAGEND, 
                    CHKA.DTHORAAGEND, CHKA.IDATIVO, CHKA.DTHRFINALAGEND, CHKA.NMCHECKLISTAGEND, CONSULOBS.DSOBSTAREFAAGEND,
                    FUNC.NMOPERADOR, LC.NMLOCALCLIENTE, CHK.NMCHECKLIST, C.NMRAZSOCCLIE, 
                    ROUND(24 *(60 *(CHKA.DTHRFINALAGEND - CHKA.DTHORAAGEND)), 2) AS DURACAO, CHKA.DSJUSTCANCELAMENTO, LC.CDCLIENTE,
                    (CASE CHKA.IDTIPOMARCACAOTAREFA
                            WHEN 'U' THEN 'Única'
                            WHEN 'M' THEN 'Múltipla'
                            ELSE NULL
                        END) AS IDTIPOMARCACAOTAREFA,
                    (SELECT RTRIM(XMLAGG(XMLELEMENT(E, FCM_TAREFA.NMTAREFA, ', ')
                                    .EXTRACT('//text()')
                                    ORDER BY FCM_TAREFA.NMTAREFA
                                ).GETCLOBVAL(), ','
                            ) AS NMTAREFA
                            FROM FCM_TAREFAAGEND
                            JOIN FCM_TAREFACHK
                            ON FCM_TAREFACHK.NRTAREFACHK = FCM_TAREFAAGEND.NRTAREFACHK
                            JOIN FCM_TAREFA 
                            ON FCM_TAREFA. NRTAREFA = FCM_TAREFACHK.NRTAREFA
                            WHERE FCM_TAREFAAGEND.NRCHECKLISTAGEND = CHKA.NRCHECKLISTAGEND
                            AND FCM_TAREFAAGEND.IDREALIZADO = 'S') AS NMTAREFAS
                        FROM FCM_CHECKLISTAGEND CHKA
                    LEFT JOIN OPERADOR FUNC 
                        ON FUNC.CDOPERADOR = CHKA.CDOPERADOR
                    LEFT JOIN FCM_LOCALCLI LC
                        ON LC.NRLOCALCLIENTE = CHKA.NRLOCALCLIENTE
                    LEFT JOIN FCM_CHECKLIST CHK
                        ON CHK.NRCHECKLIST = CHKA.NRCHECKLIST
                    LEFT JOIN CLIENTE C
                        ON C.CDCLIENTE = LC.CDCLIENTE
                    LEFT JOIN (SELECT DISTINCT DSOBSTAREFAAGEND, NRCHECKLISTAGEND
                                    FROM FCM_TAREFAAGEND) CONSULOBS
                        ON CONSULOBS.NRCHECKLISTAGEND = CHKA.NRCHECKLISTAGEND
                    WHERE ((CHKA.CDOPERADOR IN ($operadores))
                        OR (CHKA.CDOPERINCLUSAO = :CDOPERADOR))";

                return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);

    }

    public function getItensTarefas($parametros)
    {
        $nrchecklistagend = $parametros;

        $sql = "SELECT TA.NRTAREFAAGEND, TA.NRCHECKLISTAGEND, TA.CDOPERADOR, TA.DSOBSTAREFAAGEND, TA.IDREALIZADO, CA.NMCHECKLISTAGEND, TF.NMTAREFA
                    FROM FCM_TAREFAAGEND TA
                    JOIN FCM_CHECKLISTAGEND CA
                      ON CA.NRCHECKLISTAGEND = TA.NRCHECKLISTAGEND
                    JOIN FCM_TAREFACHK T
                      ON T.NRTAREFACHK = TA.NRTAREFACHK
                    JOIN FCM_TAREFA TF
                      ON T.NRTAREFA = TF.NRTAREFA
                   WHERE TA.NRCHECKLISTAGEND = $nrchecklistagend";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFotosTarefas($parametros)
    {
        $nrchecklistagend = $parametros;

        $sql = "SELECT FTA.NRFOTOTAREFAAGEND, FTA.NRTAREFAAGEND, FTA.DSFOTOTAREFAAGEND
                    FROM FCM_FOTOTAREFAAGEND FTA
                    JOIN FCM_TAREFAAGEND TA
                      ON TA.NRTAREFAAGEND = FTA.NRTAREFAAGEND
                    JOIN FCM_CHECKLISTAGEND CHKA
                      ON CHKA.NRCHECKLISTAGEND = TA.NRCHECKLISTAGEND
                   WHERE CHKA.NRCHECKLISTAGEND = $nrchecklistagend";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFiltroRelatorioLocal()
    {
        $sql = "SELECT NRLOCALCLIENTE, NMLOCALCLIENTE
                FROM FCM_LOCALCLI 
                WHERE NRTIPOLOCALCLI IN (SELECT NRTIPOLOCALCLI
                                            FROM FCM_TIPOLOCALCLI
                                        WHERE IDTIPOLOCALCLI = '3') 
                AND IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFiltroRelatorioChecklist()
    {
        $sql = "SELECT NRCHECKLIST, NMCHECKLIST
                FROM FCM_CHECKLIST
               WHERE IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFiltroRelatorioCliente()
    {
        $sql = "SELECT CDCLIENTE, NMRAZSOCCLIE 
                FROM CLIENTE
                WHERE IDCLIEATIV = 'S'
                    AND IDATIVO = 'S' 
                    AND CDCLIENTE IN (SELECT CDCLIENTE 
                                        FROM FCM_LOCALCLI
                                    WHERE IDATIVO = 'S')";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
}
