<?php

namespace FM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\ConnectionCustom;
use FM\Factory\IntegracaoSMFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;

class IntegracaoSMService
{

    protected $entityManager;
    protected $connection;
    protected $connectionCustom;
    protected $integracaoSMFactory;
    protected $Environment;
    protected $newCode;

    public function __construct(EntityManager $entityManager, ConnectionCustom $connectionCustom, Environment $Environment, NewCode $newCode)
    {

        $this->entityManager        = $entityManager;
        $this->connectionCustom     = $connectionCustom;
        $this->Environment          = $Environment;
        $this->newCode              = $newCode;
    }

    private function setConnection($nrorg)
    {
        $novoEntityManager = $this->connectionCustom->updateVpdConnection($nrorg);
        if ($novoEntityManager) {
            $this->entityManager = $novoEntityManager;
            $this->connection    = $novoEntityManager->getConnection();
        } else {
            $this->connection = $this->entityManager->getConnection();
        }
    }

    public function cadastrarLocais($dados)
    {
        try {
            $nrorg = $dados[0]['NRORG'];

            $this->setConnection($nrorg);
            $this->integracaoFactory = new IntegracaoSMFactory($this->entityManager);

            foreach ($dados as $local) {
                $params = array(
                    "CDCLIENTE"      => $local["CDCLIENTE"],
                    "NRESTRUTURAM"   => $local["NRESTRUTURAM"],
                    "NRORG"          => $local["NRORG"]
                );

                $valida = $this->integracaoFactory->validaInsertLocalCli($params);

                if ($valida == 0) {
                    $nrlocalcliente = $this->newCode->getCodeByTableName("FCM_LOCALCLI", $nrorg, 1, 10, true);
                    $params = array(
                        "NRLOCALCLIENTE" => $nrlocalcliente,
                        "NMLOCALCLIENTE" => $local["NMESTRUTURAM"],
                        "CDCLIENTE"      => $local["CDCLIENTE"],
                        "NRTIPOLOCALCLI" => $local["NRTIPOLOCALCLI"],
                        "NRESTRUTURAM"   => $local["NRESTRUTURAM"],
                        "NRORG"          => $local["NRORG"],
                        "OPERADOR"       => $local["CDOPERADOR"]
                    );

                    $this->integracaoFactory->cadastrarLocais($params);
                }
            };

            return 'OK';
        } catch (\Exception $e) {
            return $e->getMessage();
        }
    }

    public function getDadosMedicao($dados)
    {

        $retorno = array();

        try {
            $nrorg = $dados['NRORG'];
            $this->setConnection($nrorg);
            $this->integracaoFactory = new IntegracaoSMFactory($this->entityManager);

            $params = array(
                "NRSEQCONTRATO"   => $dados["NRSEQCONTRATO"],
                "NRMEDICAOFATURA" => $dados["NRMEDICAOFATURA"],
                "NRORG"           => $nrorg
            );

            $resultados = $this->integracaoFactory->getDadosMedicao($params);

            if (!empty($resultados)) {
                foreach ($resultados as $id => $resultado) {
                    $retorno['DADOS'] = array(
                        $id = array(
                            "QTMEDIDA"        => $resultado["QTMEDIDA"],
                            "NRMEDICAOFATURA" => $resultado["NRMEDICAOFATURA"],
                            "SERVICO"         => $resultado["SERVICO"],
                            "MEDIDA"          => $resultado["MEDIDA"]
                        )
                    );
                }

                $retorno['RETORNO'] = 'OK';
            } else {
                $retorno['RETORNO'] = 'Não foram encontrados dados para realização da medição.<br><br>Favor verificar os serviços e medidas parametrizados na tela "Parametrização Geral" do módulo "FM".';
            }

            return $retorno;
        } catch (\Exception $e) {
            $retorno['RETORNO'] = $e->getMessage();
            return $retorno;
        }
    }
}
